@@ -49,7 +49,7 @@ class Receiver(object):
         self.__properties = dict(properties)
 
         self.__compatible_remotes = properties.get('compatible-remotes')
-
+        
         if self.__compatible_remotes:
             self.__compatible_remotes = [
                 name.strip() for name in
@@ -197,14 +197,35 @@ class Remote(object):
 
         mechanism.WriteRemoteConfiguration(self.configuration)
 
+    def __set_contributor(self, value):
+        '''
+        Updates the contributor property of this remote.
+        '''
+        
+        self.__contributor = value
+
+    def __set_vendor(self, value):
+        '''
+        Updates the vendor property of this remote.
+        '''
+        
+        self.__vendor = value
+
+    def __set_product(self, value):
+        '''
+        Updates the product property of this remote.
+        '''
+        
+        self.__product = value
+
     def __repr__(self):
         return '<Remote: %s>' % self.__name
 
     # pylint: disable-msg=W0212
     name = property(lambda self: self.__name)
-    product = property(lambda self: self.__product)
-    vendor = property(lambda self: self.__vendor)
-    contributor = property(lambda self: self.__contributor)
+    product = property(lambda self: self.__product, __set_product)
+    vendor = property(lambda self: self.__vendor, __set_vendor)
+    contributor = property(lambda self: self.__contributor, __set_contributor)
 
     configuration = property(__get_configuration)
 
@@ -340,11 +361,28 @@ class RemotesDatabase(object):
     def find(self, vendor, product, default=None):
         '''Tries to find the specified remote control.'''
 
+        # Cope with Unknown vendors:
+        # We store 'Unknown' (not translated) in the configuration file in backend.WriteRemoteConfiguration() if vendor is None:
+        vendor_name = vendor
+        if(vendor_name == 'Unknown'):
+          vendor_name = None
+
+        # Try looking at the remote.product
+        # (the combo lists product.name if remote.product is None):
         for remote in self:
-            if (remote.vendor == vendor and
+            #print ("RemotesDatabase.find(): remote.vendor=%s, remote_product=%s\n" % (remote.vendor, remote.product))
+            if (remote.vendor == vendor_name and
                 remote.product == product):
                 return remote
 
+        # Try looking at the remote.name
+        # (the combo lists product.name if remote.product is None):
+        for remote in self:
+            #print ("RemotesDatabase.find(): remote.vendor=%s, remote_product=%s\n" % (remote.vendor, remote.product))
+            if (remote.vendor == vendor_name and
+                remote.name == product):
+                return remote
+
         return default
 
     def __iter__(self):
@@ -801,6 +839,7 @@ class KeyListener(gobject.GObject):
         # pylint: disable-msg=E1002
         super(KeyListener, self).__init__()
 
+        self.__running = False
         self.__socket_name = socket_name
         self.__reconnect_source = 0
         self.__socket_source = 0
@@ -808,17 +847,79 @@ class KeyListener(gobject.GObject):
         self.__buffer = ''
 
     def start(self):
-        '''
-        Starts listening.
-        '''
+        '''Starts listening.'''
+
+        self.__running = True
 
         if not self.__connect():
             self.__reconnect()
 
+    def stop(self):
+        '''Stops listening.'''
+
+        self.__running = False
+        self.__disconnect()
+
+    def __on_io_event(self, fd, condition):
+        '''Handle I/O events on the lircd socket.'''
+
+        logging.info('I/O event on lirc socket %d: %d', fd, condition)
+
+        if condition & gobject.IO_IN:
+            logging.info('reading from lirc socket %d...', fd)
+
+            try:
+              # TODO: What is this 128? No magic numbers in code, please.
+              packet = self.__socket and self.__socket.recv(128)
+            except SocketError, ex:
+              logging.error('__on_io_event(): socket.recv() threw exception reading from lirc socket: %s', ex.message)
+              packet = ''
+
+            logging.info('...%d bytes received.', len(packet))
+            self.__buffer += packet
+
+            while True:
+                eol = self.__buffer.find('\n')
+
+                if eol < 0:
+                    break
+
+                packet = self.__buffer[:eol]
+                self.__buffer = self.__buffer[eol + 1:]
+
+                logging.info('processing packet: %r', packet)
+
+                try:
+                    code, repeat, name, remote = packet.split()
+                    repeat = int(repeat, 16)
+                    code = long(code, 16)
+
+                    # pylint: disable-msg=E1101
+                    self.emit('key-pressed', remote, repeat, name, code)
+
+                # pylint: disable-msg=W0704
+                except ValueError:
+                    pass
+
+        if condition & gobject.IO_HUP:
+            if self.__running:
+                def restart():
+                    if self.__running:
+                        self.start()
+
+                    return False
+
+                gobject.timeout_add(1000, restart)
+
+            else:
+                self.__disconnect()
+
+            return False
+
+        return True
+
     def __connect(self):
-        '''
-        Connects to the lircd socket.
-        '''
+        '''Connects to the lircd socket.'''
 
         self.__disconnect()
 
@@ -828,54 +929,12 @@ class KeyListener(gobject.GObject):
         self.__buffer = ''
 
         try:
-            # pylint: disable-msg=W0613
-            def on_io(fd, condition):
-                '''
-                Handle I/O events on the lircd socket.
-                '''
-
-                logging.info('I/O event on lirc socket %d: %d', fd, condition)
-
-                if condition & gobject.IO_IN:
-                    logging.info('reading from lirc socket %d...', fd)
-                    packet = self.__socket and self.__socket.recv(128)
-
-                    logging.info('...%d bytes received.', len(packet))
-                    self.__buffer += packet
-
-                    while True:
-                        eol = self.__buffer.find('\n')
-
-                        if eol < 0:
-                            break
-
-                        packet = self.__buffer[:eol]
-                        self.__buffer = self.__buffer[eol + 1:]
-
-                        logging.info('processing packet: %r', packet)
-
-                        try:
-                            code, repeat, name, remote = packet.split()
-                            repeat = int(repeat, 16)
-                            code = long(code, 16)
-
-                            # pylint: disable-msg=E1101
-                            self.emit('key-pressed', remote, repeat, name, code)
-
-                        # pylint: disable-msg=W0704
-                        except ValueError:
-                            pass
-
-                if condition & gobject.IO_HUP:
-                    self.__disconnect()
-                    return False
-
-                return True
-
             self.__socket = socket(AF_UNIX)
-            self.__socket_source = gobject.io_add_watch(self.__socket.fileno(),
-                                                        gobject.IO_IN | gobject.IO_HUP,
-                                                        on_io)
+
+            self.__socket_source = (
+                gobject.io_add_watch(self.__socket.fileno(),
+                                     gobject.IO_IN | gobject.IO_HUP,
+                                     self.__on_io_event))
 
             self.__socket.connect(self.__socket_name)
 
@@ -897,9 +956,7 @@ class KeyListener(gobject.GObject):
             return False
 
     def __disconnect(self):
-        '''
-        Disconnects from the lircd socket.
-        '''
+        '''Disconnects from the lircd socket.'''
 
         self.__buffer = ''
 
@@ -917,17 +974,13 @@ class KeyListener(gobject.GObject):
         self.emit('changed')
 
     def __reconnect(self):
-        '''
-        Tries to reconnects to the lircd socket.
-        '''
+        '''Tries to reconnects to the lircd socket.'''
 
         if not self.__reconnect_source:
             self.__reconnect_source = gobject.timeout_add(5000, self._on_reconnect)
 
     def _on_reconnect(self):
-        '''
-        Called regularly when the key listener tries to reconnect.
-        '''
+        '''Called regularly when the key listener tries to reconnect.'''
 
         if self.__connect():
             self.__reconnect_source = 0
@@ -950,7 +1003,7 @@ class HardwareConfParser(object):
         self.__values = dict()
 
         for line in open(filename, 'r'):
-            tokens = [t.strip() for t in line.split('=', 2)]
+            tokens = [t.strip() for t in line.split('=', 1)]
 
             if len(tokens) < 2:
                 continue
@@ -985,27 +1038,21 @@ class HardwareConfParser(object):
         except KeyError:
             return default
 
-def has_include_keyword():
-    '''Checks if include statements are supported in lircd.conf.'''
-
-    return (config.LSB_RELEASE.check(name='Ubuntu', codename='hardy') or
-            config.LSB_RELEASE.check(name='Ubuntu', release='8.04'))
-
 def find_remote_config():
     '''Finds the location our customized lircd.conf file.'''
 
-    if has_include_keyword():
-        return config.LIRC_REMOTE_CONF, True
-
-    return config.LIRC_DAEMON_CONF, False
+    return config.LIRC_REMOTE_CONF
 
 def check_hardware_settings(selected_remote):
     '''Check if the hardware settings are sane.'''
 
-    remote_config, can_include = find_remote_config()
+    remote_config = find_remote_config()
 
-    if can_include and not re.search(
-        r'^\s*include\s+%s\s*$' % re.escape(config.LIRC_REMOTE_CONF),
+    if not os.path.exists(remote_config):
+        return True
+
+    if not re.search(
+        r'^\s*include\s+<%s>\s*$' % re.escape(config.LIRC_REMOTE_CONF),
         open(config.LIRC_DAEMON_CONF).read(), re.MULTILINE):
         return False
 
@@ -1013,6 +1060,12 @@ def check_hardware_settings(selected_remote):
     parser.read(open(remote_config))
 
     for remote in parser.remotes:
+        if remote is None:
+            return False
+
+        if selected_remote is None:
+            return False
+
         if (remote.vendor == selected_remote.vendor and
             remote.product == selected_remote.product):
             return True
